@@ -20,6 +20,24 @@ else
     AODH_BIN_DIR=$(get_python_exec_prefix)
 fi
 
+
+if [ -z "$AODH_DEPLOY" ]; then
+    # Default
+    AODH_DEPLOY=simple
+
+    # Fallback to common wsgi devstack configuration
+    if [ "$ENABLE_HTTPD_MOD_WSGI_SERVICES" == "True" ]; then
+        AODH_DEPLOY=mod_wsgi
+
+    # Deprecated config
+    elif [ -n "$AODH_USE_MOD_WSGI" ] ; then
+        echo_summary "AODH_USE_MOD_WSGI is deprecated, use AODH_DEPLOY instead"
+        if [ "$AODH_USE_MOD_WSGI" == True ]; then
+            AODH_DEPLOY=mod_wsgi
+        fi
+    fi
+fi
+
 # Test if any Aodh services are enabled
 # is_aodh_enabled
 function is_aodh_enabled {
@@ -32,7 +50,7 @@ function aodh_service_url {
 }
 
 
-# _install_mongdb - Install mongodb and pyton lib.
+# _install_mongdb - Install mongodb and python lib.
 function _aodh_install_mongodb {
     # Server package is the same on all
     local packages=mongodb-server
@@ -84,12 +102,18 @@ function _aodh_config_apache_wsgi {
     fi
 
     sudo cp $AODH_DIR/devstack/apache-aodh.template $aodh_apache_conf
+    if [ "$AODH_BACKEND" = 'hbase' ] ; then
+        # Use one process to have single in-memory DB instance for data consistency
+        AODH_API_WORKERS=1
+    else
+        AODH_API_WORKERS=$API_WORKERS
+    fi
     sudo sed -e "
         s|%PORT%|$AODH_SERVICE_PORT|g;
         s|%APACHE_NAME%|$APACHE_NAME|g;
         s|%WSGIAPP%|$AODH_WSGI_DIR/app|g;
         s|%USER%|$STACK_USER|g;
-        s|%APIWORKERS%|$API_WORKERS|g;
+        s|%APIWORKERS%|$AODH_API_WORKERS|g;
         s|%VIRTUALENV%|$venv_path|g
     " -i $aodh_apache_conf
 }
@@ -117,21 +141,24 @@ function _aodh_create_accounts {
 
         create_service_user "aodh" "admin"
 
-        if [[ "$KEYSTONE_CATALOG_BACKEND" = 'sql' ]]; then
-            local aodh_service=$(get_or_create_service "aodh" \
-                "alarming" "OpenStack Alarming Service")
-            get_or_create_endpoint $aodh_service \
-                "$REGION_NAME" \
-                "$(aodh_service_url)/" \
-                "$(aodh_service_url)/" \
-                "$(aodh_service_url)/"
-        fi
+        local aodh_service=$(get_or_create_service "aodh" \
+            "alarming" "OpenStack Alarming Service")
+        get_or_create_endpoint $aodh_service \
+            "$REGION_NAME" \
+            "$(aodh_service_url)" \
+            "$(aodh_service_url)" \
+            "$(aodh_service_url)"
     fi
 }
 
 # Activities to do before aodh has been installed.
 function preinstall_aodh {
-    echo_summary "Preinstall not in virtualenv context. Skipping."
+    # Needed to build psycopg2
+    if is_ubuntu; then
+        install_package libpq-dev
+    else
+        install_package postgresql-devel
+    fi
 }
 
 # Remove WSGI files, disable and remove Apache vhost file
@@ -146,7 +173,7 @@ function cleanup_aodh {
     if [ "$AODH_BACKEND" = 'mongodb' ] ; then
         mongo aodh --eval "db.dropDatabase();"
     fi
-    if [ "$AODH_USE_MOD_WSGI" == "True" ]; then
+    if [ "$AODH_DEPLOY" == "mod_wsgi" ]; then
         _aodh_cleanup_apache_wsgi
     fi
 }
@@ -155,10 +182,11 @@ function cleanup_aodh {
 function _aodh_configure_storage_backend {
     if [ "$AODH_BACKEND" = 'mysql' ] || [ "$AODH_BACKEND" = 'postgresql' ] ; then
         iniset $AODH_CONF database connection $(database_connection_url aodh)
-        iniset $AODH_CONF DEFAULT collector_workers $API_WORKERS
     elif [ "$AODH_BACKEND" = 'mongodb' ] ; then
         iniset $AODH_CONF database connection mongodb://localhost:27017/aodh
         cleanup_aodh
+    elif [ "$AODH_BACKEND" = 'hbase' ] ; then
+        iniset $AODH_CONF database connection hbase://__test__
     else
         die $LINENO "Unable to configure unknown AODH_BACKEND $AODH_BACKEND"
     fi
@@ -168,47 +196,76 @@ function _aodh_configure_storage_backend {
 function configure_aodh {
     iniset_rpc_backend aodh $AODH_CONF
 
-    iniset $AODH_CONF DEFAULT notification_topics "$AODH_NOTIFICATION_TOPICS"
-    iniset $AODH_CONF DEFAULT verbose True
+    iniset $AODH_CONF oslo_messaging_notifications topics "$AODH_NOTIFICATION_TOPICS"
     iniset $AODH_CONF DEFAULT debug "$ENABLE_DEBUG_LOG_LEVEL"
 
     if [[ -n "$AODH_COORDINATION_URL" ]]; then
         iniset $AODH_CONF coordination backend_url $AODH_COORDINATION_URL
     fi
 
+    # Set up logging
+    if [ "$SYSLOG" != "False" ]; then
+        iniset $AODH_CONF DEFAULT use_syslog "True"
+    fi
+
+    # Format logging
+    if [ "$LOG_COLOR" == "True" ] && [ "$SYSLOG" == "False" ] && [ "$AODH_DEPLOY" != "mod_wsgi" ]; then
+        setup_colorized_logging $AODH_CONF DEFAULT
+    fi
+
     # Install the policy file for the API server
     cp $AODH_DIR/etc/aodh/policy.json $AODH_CONF_DIR
     iniset $AODH_CONF oslo_policy policy_file $AODH_CONF_DIR/policy.json
 
     cp $AODH_DIR/etc/aodh/api_paste.ini $AODH_CONF_DIR
+    cp $AODH_DIR/etc/aodh/aodh-config-generator.conf $AODH_CONF_DIR
 
-    # The alarm evaluator needs these options to call aodh APIs
-    iniset $AODH_CONF service_credentials os_username aodh
-    iniset $AODH_CONF service_credentials os_password $SERVICE_PASSWORD
-    iniset $AODH_CONF service_credentials os_tenant_name $SERVICE_TENANT_NAME
-    iniset $AODH_CONF service_credentials os_region_name $REGION_NAME
-    iniset $AODH_CONF service_credentials os_auth_url $KEYSTONE_SERVICE_URI/v2.0
+    # The alarm evaluator needs these options to call gnocchi/ceilometer APIs
+    iniset $AODH_CONF service_credentials auth_type password
+    iniset $AODH_CONF service_credentials username aodh
+    iniset $AODH_CONF service_credentials user_domain_id default
+    iniset $AODH_CONF service_credentials project_domain_id default
+    iniset $AODH_CONF service_credentials password $SERVICE_PASSWORD
+    iniset $AODH_CONF service_credentials project_name $SERVICE_PROJECT_NAME
+    iniset $AODH_CONF service_credentials region_name $REGION_NAME
+    iniset $AODH_CONF service_credentials auth_url $KEYSTONE_SERVICE_URI
 
-    # TODO(chdent): Until
-    # https://bugs.launchpad.net/keystonemiddleware/+bug/1482078
-    # and
-    # https://bugs.launchpad.net/keystonemiddleware/+bug/1406218
-    # are resolved the easiest way to deal with the auth_token
-    # middleware when using a non-global conf is to put it in the
-    # paste file.
-    configure_auth_token_middleware $AODH_CONF_DIR/api_paste.ini aodh $AODH_AUTH_CACHE_DIR filter:authtoken
-
-    iniset $AODH_CONF notification store_events $AODH_EVENTS
+    configure_auth_token_middleware $AODH_CONF aodh $AODH_AUTH_CACHE_DIR
 
     # Configured storage
     _aodh_configure_storage_backend
 
     # NOTE: This must come after database configuration as those can
     # call cleanup_aodh which will wipe the WSGI config.
-    if [ "$AODH_USE_MOD_WSGI" == "True" ]; then
+    if [ "$AODH_DEPLOY" == "mod_wsgi" ]; then
         iniset $AODH_CONF api pecan_debug "False"
         _aodh_config_apache_wsgi
+    elif [ "$AODH_DEPLOY" == "uwsgi" ]; then
+        # iniset creates these files when it's called if they don't exist.
+        AODH_UWSGI_FILE=$AODH_CONF_DIR/aodh-uwsgi.ini
+
+        rm -f "$AODH_UWSGI_FILE"
+
+        iniset "$AODH_UWSGI_FILE" uwsgi http $AODH_SERVICE_HOST:$AODH_SERVICE_PORT
+        iniset "$AODH_UWSGI_FILE" uwsgi wsgi-file "$AODH_DIR/aodh/api/app.wsgi"
+        # This is running standalone
+        iniset "$AODH_UWSGI_FILE" uwsgi master true
+        # Set die-on-term & exit-on-reload so that uwsgi shuts down
+        iniset "$AODH_UWSGI_FILE" uwsgi die-on-term true
+        iniset "$AODH_UWSGI_FILE" uwsgi exit-on-reload true
+        iniset "$AODH_UWSGI_FILE" uwsgi threads 10
+        iniset "$AODH_UWSGI_FILE" uwsgi processes $API_WORKERS
+        iniset "$AODH_UWSGI_FILE" uwsgi enable-threads true
+        iniset "$AODH_UWSGI_FILE" uwsgi plugins python
+        iniset "$AODH_UWSGI_FILE" uwsgi lazy-apps true
+        # uwsgi recommends this to prevent thundering herd on accept.
+        iniset "$AODH_UWSGI_FILE" uwsgi thunder-lock true
+        # Override the default size for headers from the 4k default.
+        iniset "$AODH_UWSGI_FILE" uwsgi buffer-size 65535
+        # Make sure the client doesn't try to re-use the connection.
+        iniset "$AODH_UWSGI_FILE" uwsgi add-header "Connection: close"
     fi
+
 }
 
 # init_aodh() - Initialize etc.
@@ -236,30 +293,38 @@ function install_aodh {
     _aodh_prepare_coordination
     _aodh_prepare_storage_backend
     install_aodhclient
-    setup_develop $AODH_DIR
-    sudo install -d -o $STACK_USER -m 755 $AODH_CONF_DIR $AODH_API_LOG_DIR
+    sudo -H pip install -e "$AODH_DIR"[test,$AODH_BACKEND]
+    sudo install -d -o $STACK_USER -m 755 $AODH_CONF_DIR
+
+    if [ "$AODH_DEPLOY" == "mod_wsgi" ]; then
+        install_apache_wsgi
+    elif [ "$AODH_DEPLOY" == "uwsgi" ]; then
+        pip_install uwsgi
+    fi
 }
 
 # install_aodhclient() - Collect source and prepare
 function install_aodhclient {
-    if use_library_from_git "python-ceilometerclient"; then
-        git_clone_by_name "python-ceilometerclient"
-        setup_dev_lib "python-ceilometerclient"
-        sudo install -D -m 0644 -o $STACK_USER {${GITDIR["python-ceilometerclient"]}/tools/,/etc/bash_completion.d/}ceilometer.bash_completion
+    if use_library_from_git "python-aodhclient"; then
+        git_clone_by_name "python-aodhclient"
+        setup_dev_lib "python-aodhclient"
     else
-        pip_install_gr python-ceilometerclient
+        pip_install_gr aodhclient
     fi
+    aodh complete | sudo tee /etc/bash_completion.d/aodh.bash_completion > /dev/null
 }
 
 # start_aodh() - Start running processes, including screen
 function start_aodh {
-    if [[ "$AODH_USE_MOD_WSGI" == "False" ]]; then
-        run_process aodh-api "$AODH_BIN_DIR/aodh-api -d -v --log-dir=$AODH_API_LOG_DIR --config-file $AODH_CONF"
-    else
+    if [[ "$AODH_DEPLOY" == "mod_wsgi" ]]; then
         enable_apache_site aodh
         restart_apache_server
         tail_log aodh /var/log/$APACHE_NAME/aodh.log
         tail_log aodh-api /var/log/$APACHE_NAME/aodh_access.log
+    elif [ "$AODH_DEPLOY" == "uwsgi" ]; then
+        run_process aodh-api "$AODH_BIN_DIR/uwsgi $AODH_UWSGI_FILE"
+    else
+        run_process aodh-api "$AODH_BIN_DIR/aodh-api -p $AODH_SERVICE_PORT"
     fi
 
     # Only die on API if it was actually intended to be turned on
@@ -277,7 +342,7 @@ function start_aodh {
 
 # stop_aodh() - Stop running processes
 function stop_aodh {
-    if [ "$AODH_USE_MOD_WSGI" == "True" ]; then
+    if [ "$AODH_DEPLOY" == "mod_wsgi" ]; then
         disable_apache_site aodh
         restart_apache_server
     fi
